@@ -1,5 +1,7 @@
 //! BGP attribute structs
+use std::fmt::{Display, Formatter};
 use std::net::IpAddr;
+use itertools::Itertools;
 use crate::network::*;
 
 /// The high-order bit (bit 0) of the Attribute Flags octet is the
@@ -58,7 +60,7 @@ pub enum AttributeFlagsBit {
 ///
 /// All attributes: <https://www.iana.org/assignments/bgp-parameters/bgp-parameters.xhtml#bgp-parameters-2>
 #[allow(non_camel_case_types)]
-#[derive(Debug, Primitive, PartialEq, Eq, Hash, Clone)]
+#[derive(Debug, Primitive, PartialEq, Eq, Hash, Copy, Clone)]
 pub enum AttrType {
     RESERVED = 0,
     ORIGIN = 1,
@@ -86,7 +88,7 @@ pub enum AttrType {
 }
 
 #[allow(non_camel_case_types)]
-#[derive(Debug, Primitive, PartialEq, Eq, Hash, Clone)]
+#[derive(Debug, Primitive, PartialEq, Eq, Hash, Copy, Clone)]
 pub enum Origin {
     IGP = 0,
     EGP = 1,
@@ -94,14 +96,14 @@ pub enum Origin {
 }
 
 #[allow(non_camel_case_types)]
-#[derive(Debug, Primitive, PartialEq, Eq, Hash, Clone)]
+#[derive(Debug, Primitive, PartialEq, Eq, Hash, Copy, Clone)]
 pub enum AtomicAggregate {
     NAG = 0,
     AG = 1,
 }
 
 /// The `Attribute` enum represents different kinds of Attribute values.
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Clone)]
 pub enum Attribute {
     Origin(Origin),
     AsPath(AsPath),
@@ -173,7 +175,6 @@ impl AsPath {
         self.segments.iter().map(AsPathSegment::count_asns).sum()
     }
 
-
     /// Construct AsPath from AS_PATH and AS4_PATH
     ///
     /// https://datatracker.ietf.org/doc/html/rfc6793#section-4.2.3
@@ -221,13 +222,31 @@ impl AsPath {
 
         Some(AsPath{ segments: new_segs })
     }
+
+    pub fn get_origin(&self) -> Option<Vec<Asn>> {
+        if let Some(seg) = self.segments.last() {
+            match seg {
+                AsPathSegment::AsSequence(v) => {
+                    if let Some(n) = v.last() {
+                        Some(vec![n.clone()])
+                    } else {
+                        None
+                    }
+                }
+                AsPathSegment::AsSet(v) => { Some(v.clone()) }
+                AsPathSegment::ConfedSequence(_) | AsPathSegment::ConfedSet(_) => { None }
+            }
+        } else {
+            None
+        }
+    }
 }
 
 /////////////////
 // COMMUNITIES //
 /////////////////
 
-#[derive(Debug, PartialEq, Clone)]
+#[derive(Debug, PartialEq, Copy, Clone)]
 pub enum Community {
     NoExport,
     NoAdvertise,
@@ -235,7 +254,7 @@ pub enum Community {
     Custom(Asn, u16),
 }
 
-#[derive(Debug, PartialEq, Clone)]
+#[derive(Debug, PartialEq, Clone, Copy)]
 pub struct LargeCommunity {
     global_administrator: u32,
     local_data: [u32; 2],
@@ -254,7 +273,7 @@ impl LargeCommunity {
 // NLRI //
 //////////
 
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Clone)]
 pub struct Nlri {
     pub afi: Afi,
     pub safi: Safi,
@@ -262,7 +281,7 @@ pub struct Nlri {
     pub prefixes: Vec<NetworkPrefix>,
 }
 
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Clone)]
 pub struct MpReachableNlri {
     afi: Afi,
     safi: Safi,
@@ -286,12 +305,12 @@ impl MpReachableNlri {
     }
 }
 
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Copy, Clone)]
 pub struct MpReachableNlriV2 {
     next_hop: NextHopAddress,
 }
 
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Clone)]
 pub struct MpUnreachableNlri {
     afi: Afi,
     safi: Safi,
@@ -308,10 +327,92 @@ impl MpUnreachableNlri {
     }
 }
 
+///////////////////
+// DISPLAY IMPLS //
+///////////////////
+
+impl Display for Origin {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        let s = match self {
+            Origin::IGP => {"IGP"}
+            Origin::EGP => {"EGP"}
+            Origin::INCOMPLETE => {"INCOMPLETE"}
+        };
+        write!(f, "{}", s)
+    }
+}
+
+impl Display for AtomicAggregate {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}", match self {
+            AtomicAggregate::NAG => {"NAG"}
+            AtomicAggregate::AG => {"AG"}
+        })
+    }
+}
+
+const NOEXPORT: &str ="no-export";
+
+impl Display for Community {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}", match self {
+            Community::NoExport => {
+                "no-export".to_string()
+            }
+            Community::NoAdvertise => {
+                "no-advertise".to_string()
+            }
+            Community::NoExportSubConfed => {
+                "no-export-sub-confed".to_string()
+            }
+            Community::Custom(asn, value) => {
+                format!("{}:{}", asn, value)
+            }
+        }
+        )
+    }
+}
+
+impl Display for NextHopAddress {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}",
+               match self {
+                   NextHopAddress::Ipv4(v) => {v.to_string()}
+                   NextHopAddress::Ipv6(v) => {v.to_string()}
+                   NextHopAddress::Ipv6LinkLocal(v1, _v2) => {v1.to_string()}
+               }
+        )
+    }
+}
+
+impl Display for AsPath {
+    fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
+        write!(f, "{}",
+               self
+                   .segments()
+                   .iter()
+                   .map(|seg| match seg {
+                       AsPathSegment::AsSequence(v) | AsPathSegment::ConfedSequence(v) => v
+                           .iter()
+                           .join(" "),
+                       AsPathSegment::AsSet(v) | AsPathSegment::ConfedSet(v) => {
+                           format!(
+                               "{{{}}}",
+                               v.iter()
+                                   .join(",")
+                           )
+                       }
+                   })
+                   .join(" ")
+        )
+    }
+}
+
+
 
 #[cfg(test)]
 mod tests {
-    use super::*;
+    use crate::bgp::attributes::{AsPath, AsPathSegment};
 
     #[test]
     fn test_aspath_as4path_merge() {
@@ -324,4 +425,40 @@ mod tests {
         let newpath = AsPath::merge_aspath_as4path(&aspath, &as4path).unwrap();
         assert_eq!(newpath.segments[0], AsPathSegment::AsSequence([1,2,3,7].to_vec()));
     }
-}
\ No newline at end of file
+
+    #[test]
+    fn test_get_origin() {
+        let aspath = AsPath{
+            segments: vec![
+                AsPathSegment::AsSequence([1,2,3,5].to_vec()),
+            ]
+        };
+        let origins = aspath.get_origin();
+        assert!(origins.is_some());
+        assert_eq!(origins.unwrap(), vec![5]);
+
+        let aspath = AsPath{
+            segments: vec![
+                AsPathSegment::AsSequence([1,2,3,5].to_vec()),
+                AsPathSegment::AsSet([7,8].to_vec())
+            ]
+        };
+        let origins = aspath.get_origin();
+        assert!(origins.is_some());
+        assert_eq!(origins.unwrap(), vec![7,8]);
+    }
+
+    #[test]
+    fn test_aspath_display() {
+        let aspath = AsPath{
+            segments: vec![AsPathSegment::AsSequence([1,2,3,5].to_vec())]
+        };
+        let mut paths = vec![];
+        for _ in 0..1000000{
+            paths.push(aspath.clone());
+        }
+        for p in paths{
+            p.to_string();
+        }
+    }
+}
