@@ -17,6 +17,8 @@ RFCs. Here is a list of them:
 pub mod bgp;
 pub mod network;
 pub mod mrt;
+pub mod prelude;
+pub mod err;
 
 #[macro_use]
 extern crate enum_primitive_derive;
\ No newline at end of file
