@@ -2,7 +2,9 @@
 
 use std::fmt::{Display, Formatter};
 use std::net::{Ipv4Addr, Ipv6Addr};
+use std::str::FromStr;
 use ipnetwork::IpNetwork;
+use crate::err::BgpModelsError;
 
 /// Meta information for an address/prefix.
 ///
@@ -50,7 +52,7 @@ pub enum Safi {
 /// enum that represents the type of the next hop address.
 ///
 /// [NextHopAddress] is used when parsing for next hops in [Nlri].
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Copy, Clone)]
 pub enum NextHopAddress {
     Ipv4(Ipv4Addr),
     Ipv6(Ipv6Addr),
@@ -64,6 +66,20 @@ pub struct NetworkPrefix {
     pub path_id: u32,
 }
 
+impl FromStr for NetworkPrefix {
+    type Err = BgpModelsError;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        let prefix = IpNetwork::from_str(s)?;
+        Ok(
+            NetworkPrefix{
+                prefix,
+                path_id: 0,
+            }
+        )
+    }
+}
+
 impl NetworkPrefix {
     pub fn new(prefix: IpNetwork, path_id: u32) -> NetworkPrefix {
         NetworkPrefix { prefix, path_id }
